@@ -5,11 +5,7 @@
 #ifndef	_BOOLEAN_H_
 #define	_BOOLEAN_H_
 
-#include "asn1/asn1c/asn_application.h"
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include "asn1/asn1c/constr_TYPE.h"
 
 /*
  * The underlying integer may contain various values, but everything
@@ -26,18 +22,9 @@ asn_struct_print_f BOOLEAN_print;
 asn_struct_compare_f BOOLEAN_compare;
 ber_type_decoder_f BOOLEAN_decode_ber;
 der_type_encoder_f BOOLEAN_encode_der;
-oer_type_decoder_f BOOLEAN_decode_oer;
-oer_type_encoder_f BOOLEAN_encode_oer;
-per_type_decoder_f BOOLEAN_decode_uper;
-per_type_encoder_f BOOLEAN_encode_uper;
-xer_type_decoder_f BOOLEAN_decode_xer;
+json_type_encoder_f BOOLEAN_encode_json;
 xer_type_encoder_f BOOLEAN_encode_xer;
-asn_random_fill_f  BOOLEAN_random_fill;
 
 #define BOOLEAN_constraint     asn_generic_no_constraint
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif	/* _BOOLEAN_H_ */
