@@ -6,11 +6,7 @@
 #ifndef	ASN_IOC_H
 #define	ASN_IOC_H
 
-#include "asn1/asn1c/asn_system.h"		/* Platform-specific types */
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include "asn1/asn1c/asn_system.h"
 
 struct asn_TYPE_descriptor_s;
 struct asn_ioc_cell_s;
@@ -42,9 +38,4 @@ typedef struct asn_ioc_cell_s {
     } open_type;
 } asn_ioc_cell_t;
 
-
-#ifdef __cplusplus
-}
-#endif
-
 #endif	/* ASN_IOC_H */
