@@ -6,12 +6,12 @@
 
 struct signed_object {
 	struct ContentInfo *cinfo;
-	struct signed_data sdata;
+	struct SignedData *sdata;
 };
 
 int signed_object_decode(struct signed_object *, struct rpki_uri *);
 int signed_object_validate(struct signed_object *, struct oid_arcs const *,
-    struct signed_object_args *);
+    struct ee_cert *);
 void signed_object_cleanup(struct signed_object *);
 
 #endif /* SRC_OBJECT_SIGNED_OBJECT_H_ */
